<?php

/*
 * Copyright (c) SocialLOFT LLC
 * mooSocial - The Web 2.0 Social Network Software
 * @website: http://www.moosocial.com
 * @author: mooSocial
 * @license: https://moosocial.com/license/
 */

require_once APP_PATH . DS . 'Plugin' . DS . 'UploadVideo' . DS . 'vendor' . DS . 'vimeo' . DS . 'autoload.php';

class UploadVideosController extends UploadVideoAppController {

    public $paginate = array(
        'limit' => RESULTS_LIMIT,
        'findType' => 'translated',
    );

    public function beforeFilter() {
        parent::beforeFilter();
        $this->loadModel('Video.Video');
        $this->loadModel('Category');
        $this->loadModel('Friend');
        $this->loadModel('Tag');
        $this->loadModel('Setting');
        $this->loadModel('SettingGroup');
    }

    public function index() {
        
    }

    public function ajax_upload() {
        $this->_checkPermission(array('aco' => 'video_upload'));
        $categories = $this->Category->getCategoriesList('Video');

        $this->set('categories', $categories);
    }

    public function ajax_upload_group($group_id = null) {
        if (empty($group_id)) {
            exit;
        }

        $this->loadModel('Group.Group');
        $aGroup = $this->Group->findById($group_id);
        $this->_checkExistence($aGroup);

        $this->_checkPermission(array('aco' => 'video_upload'));
        $categories = $this->Category->getCategoriesList('Video');

        $this->set(compact('group_id', 'categories'));
    }

    public function process_upload() {
        $this->autoRender = false;
        $this->_checkPermission(array('confirm' => true));
        
        $videoSizeLimit = $this->_getVideoSizeLimit();
        $videoExtensions = MooCore::getInstance()->_getVideoAllowedExtension();

        App::import('Vendor', 'qqFileUploader');
        $uploader = new qqFileUploader($videoExtensions, $videoSizeLimit);

        $path = 'uploads' . DS . 'tmp';
        $result = $uploader->handleUpload(WWW_ROOT . $path);
        echo htmlspecialchars(json_encode($result), ENT_NOQUOTES);
    }

    public function save_upload() {
        $this->autoRender = false;
        $uid = $this->Auth->user('id');
        $this->_checkPermission(array('confirm' => true));

        $this->request->data['user_id'] = $uid;
        $this->request->data['in_process'] = 1;
        $this->request->data['pc_upload'] = 1;

        if (!empty($this->request->data['group_id'])) {
            // find group
            $this->loadModel('Group.Group');
            $aGroup = $this->Group->findById($this->request->data['group_id']);
            $iGroupPrivacy = isset($aGroup['Group']['type']) ? $aGroup['Group']['type'] : PRIVACY_EVERYONE;

            $this->request->data['category_id'] = 0;
            $this->request->data['type'] = 'Group_Group';
            $this->request->data['target_id'] = $this->request->data['group_id'];
            $this->request->data['privacy'] = ($iGroupPrivacy == 3) ? 1 : $iGroupPrivacy;
        }

        //credit hook
        $event = new CakeEvent('Plugin.Controller.Video.beforeSaveVideo', $this, array(
            'destination' => $this->request->data['destination']
        ));

        $this->getEventManager()->dispatch($event);

        if(isset($event->result['status']) && !$event->result['status']){
            echo json_encode(array('result' => false, 'message' => $event->result['msg']));
            exit();
        }
        //end credit hook

        $this->Video->set($this->request->data);

        // validate data
        $this->_validateData($this->Video);
        $this->Video->Behaviors->disable('Activity');
        if ($this->Video->save()) { // successfully saved   
            // Prepare dir
            $tmpPath = WWW_ROOT . 'uploads' . DS . 'tmp';
            $videoPath = WWW_ROOT . 'uploads' . DS . 'videos' . DS . 'thumb' . DS . $this->Video->id;

            if (!file_exists($videoPath)) {
                mkdir($videoPath, 0755, true);
                file_put_contents($videoPath . DS . 'index.html', '');
            }

            $cakeEvent = new CakeEvent('Plugin.Controller.Video.afterSave', $this, array(
                'id' => $this->Video->id,
                'uid' => $uid,
                'privacy' => $this->request->data['privacy']
            ));
            $this->getEventManager()->dispatch($cakeEvent);

            // move video from tmp path to video plugin folder
            rename($tmpPath . DS . $this->request->data['destination'], $videoPath . DS . $this->request->data['destination']);

            $response['result'] = 1;
            $response['id'] = $this->Video->id;
            echo json_encode($response);
        }
    }

    // convert when upload video on feed
    public function feed_convert() {

        $this->autoRender = false;
        $activity_id = isset($this->request->data['activity_id']) ? $this->request->data['activity_id'] : 0;

        if (empty($activity_id)) {
            exit;
        }

        $this->loadModel('Activity');
        $uid = MooCore::getInstance()->getViewer(true);
        $activity = $this->Activity->findById($activity_id);
        $params = json_decode($activity['Activity']['params'], true);

        // starting convert video
        $data = array();
        $data['user_id'] = $uid;
        $data['in_process'] = 1;
        $data['pc_upload'] = 1;
        $data['title'] = $params['title'];
        $data['category_id'] = $params['category_id'];
        $data['description'] = $params['description'];
        $data['destination'] = $params['video_destination'];
        $data['privacy'] = $activity['Activity']['privacy'];

        // fixed MOOSOCIAL-2673
        if ($activity['Activity']['type'] == 'Group_Group') {
            $groupModel = MooCore::getInstance()->getModel('Group.Group');
            $group = $groupModel->findById($activity['Activity']['target_id']);
            $data['privacy'] = $group['Group']['moo_privacy'];
            $data['group_id'] = $activity['Activity']['target_id'];
        }

        if ($activity['Activity']['type'] == 'Event_Event') {
            $eventModel = MooCore::getInstance()->getModel('Event.Event');
            $event = $eventModel->findById($activity['Activity']['target_id']);
            $data['privacy'] = $event['Event']['moo_privacy'];
            $data['group_id'] = $activity['Activity']['target_id'];
        }

        $this->Video->set($data);
        $this->Video->Behaviors->disable('Activity');
        if ($this->Video->save()) { // successfully saved   
            $aVideo = $this->Video->read();

            // move uploaded file from tmp to video folder
            $tmpPath = 'uploads' . DS . 'tmp';
            $videoPath = 'uploads' . DS . 'videos' . DS . 'thumb' . DS . $aVideo['Video']['id'];

            // Prepare dir
            $tmpPath = WWW_ROOT . $tmpPath;
            $videoPath = WWW_ROOT . $videoPath;

            if (!file_exists($videoPath)) {
                mkdir($videoPath, 0755, true);
                file_put_contents($videoPath . DS . 'index.html', '');
            }

            // move video from tmp path to video plugin folder
            rename($tmpPath . DS . $aVideo['Video']['destination'], $videoPath . DS . $aVideo['Video']['destination']);
            $uploadType = Configure::read('UploadVideo.vimeo_upload') ? 'vimeo' : 'convert';

            if ($uploadType == 'convert') {
                $data = array(
                    'Activity.status' => '"' . ACTIVITY_OK . '"',
                    'Activity.item_id' => $aVideo['Video']['id']
                );
                $result = $this->feed_convert_ffmpeg($aVideo);
            } else {
                $data = array(
                    'Activity.item_id' => $aVideo['Video']['id']
                );
                $result = $this->vimeo($aVideo['Video']['id']);
            }

            $this->Activity->clear();
            $this->Activity->updateAll($data, array('Activity.id' => $activity_id));

            return $result;
        }
    }

    // convert video to valid format
    public function convert() {
        // disable render layout
        $this->autoRender = false;
        if (empty($this->request->data['video_id'])) {
            exit;
        }

        $aVideo = $this->Video->findById($this->request->data['video_id']);
        $this->_checkExistence($aVideo);

        if ($this->convert_ffmpeg($aVideo)) {
            echo json_encode(array('result' => 1));
        } else {
            echo json_encode(array('result' => 0, 'message' => __('Error while converting.')));
        }
    }

    public function vimeo($param_video_id = "") {
        // disable render layout
        $this->autoRender = false;
        $video_id = isset($this->request->data['video_id']) ? $this->request->data['video_id'] : 0;

        if ($param_video_id != "") {
            $video_id = $param_video_id;
        }

        if (empty($video_id)) {
            exit;
        }

        $aVideo = $this->Video->findById($video_id);
        $this->_checkExistence($aVideo);

        // check file size
        $videoPath = 'uploads' . DS . 'videos' . DS . 'thumb' . DS . $video_id . DS . $aVideo['Video']['destination'];
        $videoPath = WWW_ROOT . $videoPath;
        $result = $this->upload_vimeo($aVideo);

        if ($result) {
            echo json_encode(array('result' => 1));
        } else {
            echo json_encode(array('result' => 0, 'message' => __('Error while converting.')));
        }
    }

    public function admin_index() {
        $cond = array();
        if (!empty($this->request->data['keyword'])) {
            $cond['MATCH(Video.title) AGAINST(? IN BOOLEAN MODE)'] = $this->request->data['keyword'];
        }

        $cond['Video.pc_upload'] = 1;
        $videos = $this->paginate('Video', $cond);
        $categories = $this->Category->getCategoriesList('Video');

        $this->set('videos', $videos);
        $this->set('categories', $categories);
        $this->set('title_for_layout', __d('upload_video', 'Upload Videos Manager'));
    }

    public function admin_delete($id = null) {
        $this->_checkPermission(array('super_admin' => 1));

        if (!empty($_POST['videos'])) {
            $videos = $this->Video->findAllById($_POST['videos']);

            foreach ($videos as $video) {
                $this->Video->deleteVideo($video);

                $cakeEvent = new CakeEvent('Plugin.Controller.Video.afterDeleteVideo', $this, array('item' => $video));
                $this->getEventManager()->dispatch($cakeEvent);
            }

            $this->Session->setFlash(__('Videos have been deleted'), 'default', array('class' => 'Metronic-alerts alert alert-success fade in'));
        }

        $this->redirect(array(
            'plugin' => 'upload_video',
            'controller' => 'upload_videos',
            'action' => 'admin_index'
        ));
    }

    private function _getVideoSizeLimit() {
        $max_upload = $this->__return_bytes(ini_get('upload_max_filesize'));
        //select post limit
        $max_post = $this->__return_bytes(ini_get('post_max_size'));
        //select memory limit
        $memory_limit = $this->__return_bytes(ini_get('memory_limit'));

        $upload_setting = Configure::read('UploadVideo.video_common_setting_max_upload') . 'M';
        $max_upload_video = $this->__return_bytes($upload_setting);

        return min($max_upload, $max_post, $memory_limit, $max_upload_video);
    }

    private function __return_bytes($val) {
        $val = trim($val);
        $last = strtolower($val[strlen($val) - 1]);
        $number = substr($val, 0, -1);
        switch ($last) {
            case 'g':
                return $number * pow(1024, 3);
            case 'm':
                return $number * pow(1024, 2);
            case 'k':
                return $number * 1024;
            default:
                return $val;
        }
    }

}
